@@ -1,7 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
    Copyright (C) 1991-1994, 2004, 2006-2007, 2009-2014, 2016-2017,
-   2020-2021 Free Software Foundation, Inc.
+   2020-2025 Free Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -133,7 +133,7 @@ debug_set_file (FILE *fp)
   if (debug != NULL && debug != stderr && debug != stdout
       && close_stream (debug) != 0)
     {
-      M4ERROR ((warning_status, errno, "error writing to debug stream"));
+      M4ERROR ((warning_status, errno, _("error writing to debug stream")));
       retcode = EXIT_FAILURE;
     }
   debug = fp;
@@ -154,7 +154,7 @@ debug_set_file (FILE *fp)
           if (debug != stderr && close_stream (debug) != 0)
             {
               M4ERROR ((warning_status, errno,
-                        "error writing to debug stream"));
+                        _("error writing to debug stream")));
               retcode = EXIT_FAILURE;
             }
           debug = stdout;
@@ -186,8 +186,7 @@ debug_flush_files (void)
      this attempt.  The stdio-safer module and friends are essential,
      so that if stdin was closed, this lseek is not on some other file
      that we have since opened.  */
-  if (lseek (STDIN_FILENO, 0, SEEK_CUR) >= 0
-      && fflush (stdin) == 0)
+  if (lseek (STDIN_FILENO, 0, SEEK_CUR) >= 0 && fflush (stdin) == 0)
     {
       fseeko (stdin, 0, SEEK_CUR);
     }
@@ -228,12 +227,12 @@ debug_message_prefix (void)
 {
   xfprintf (debug, "m4debug:");
   if (current_line)
-  {
-    if (debug_level & DEBUG_TRACE_FILE)
-      xfprintf (debug, "%s:", current_file);
-    if (debug_level & DEBUG_TRACE_LINE)
-      xfprintf (debug, "%d:", current_line);
-  }
+    {
+      if (debug_level & DEBUG_TRACE_FILE)
+        xfprintf (debug, "%s:", current_file);
+      if (debug_level & DEBUG_TRACE_LINE)
+        xfprintf (debug, "%d:", current_line);
+    }
   putc (' ', debug);
 }
 
@@ -290,7 +289,7 @@ trace_format (const char *fmt, ...)
 
         case 'd':
           d = va_arg (args, int);
-          s = ntoa (d, 10);
+          s = ntoa (d, 10, NULL);
           break;
 
         default:
@@ -346,10 +345,10 @@ trace_flush (void)
   obstack_free (&trace, line);
 }
 
-/*-------------------------------------------------------------.
-| Do pre-argument-collction tracing for macro NAME.  Used from |
-| expand_macro ().                                             |
-`-------------------------------------------------------------*/
+/*--------------------------------------------------------------.
+| Do pre-argument-collection tracing for macro NAME.  Used from |
+| expand_macro ().                                              |
+`--------------------------------------------------------------*/
 
 void
 trace_prepre (const char *name, int id)
